//  ======================================================================== //
//                                                                           //
//    File      : Bmp.cpp                                                    //
//    Purpose   :                                                            //
//    Time-stamp: <Fri Nov/29/2002 13:45 MET Coder@ReallySoft.de>            //
//                                                                           //
//    (C) February 2002 by Ralf Westram                                      //
//                                                                           //
//    Permission to use, copy, modify, distribute and sell this software     //
//    and its documentation for any purpose is hereby granted without fee,   //
//    provided that the above copyright notice appear in all copies and      //
//    that both that copyright notice and this permission notice appear      //
//    in supporting documentation.                                           //
//                                                                           //
//    Ralf Westram makes no representations about the suitability of this    //
//    software for any purpose.  It is provided "as is" without express or   //
//    implied warranty.                                                      //
//                                                                           //
//  ======================================================================== //

#include "Bmp.h"

#include <cstdio>
#include <Err.h>
#include <Tools.h>


using namespace std;
using namespace rs;
using namespace rs::err;
using namespace rs::str;

// start of implementation of class Pixel:

unsigned char Pixel::transparent_red       = 0;
unsigned char Pixel::transparent_green     = 0;
unsigned char Pixel::transparent_blue      = 0;
bool          Pixel::use_transparent_color = false;

// -------------------------------------------------------------------------------------------------------
//      void Pixel::defineTransparentColor(unsigned char red, unsigned char green, unsigned char blue)
// -------------------------------------------------------------------------------------------------------
void Pixel::defineTransparentColor(unsigned char red, unsigned char green, unsigned char blue)
{
    transparent_red   = red;
    transparent_blue  = blue;
    transparent_green = green;
}

// -end- of implementation of class Pixel.

// start of implementation of class Bmp:

#define BUFFERSIZE 20
// #define VERBOSE

// -----------------------------------------------
//      Bmp::Bmp(size_t xsize_, size_t ysize_)
// -----------------------------------------------
Bmp::Bmp(size_t xsize_, size_t ysize_)
    : loaded_from_file("")
    , xsize(xsize_)
    , ysize(ysize_)
    , offset_per_line(xsize*3)
{
    hassert(xsize>0 && ysize>0);

    size_t datasize = ysize*offset_per_line;

    data = new unsigned char[datasize];
    memset(data, 0, datasize);
}

// -----------------------------------
//      Bmp::Bmp(const Bmp& other)
// -----------------------------------
Bmp::Bmp(const Bmp& other)
    : loaded_from_file(other.loaded_from_file)
    , xsize(other.xsize)
    , ysize(other.ysize)
    , offset_per_line(other.offset_per_line)
{

    size_t datasize = ysize*offset_per_line;

    data = new unsigned char[datasize];
    memcpy(data, other.data, datasize);
}

// -------------------------------------------------
//      Bmp& Bmp::operator = (const Bmp& other)
// -------------------------------------------------
Bmp& Bmp::operator = (const Bmp& other) {
    if (this != &other) {
        delete data;

        loaded_from_file = other.loaded_from_file;
        xsize            = other.xsize;
        ysize            = other.ysize;
        offset_per_line  = other.offset_per_line;

        size_t datasize = ysize*offset_per_line;

        data = new unsigned char[datasize];
        memcpy(data, other.data, datasize);
    }

    return *this;
}

// ----------------------------------------------
//      Bmp::Bmp(const std::string& filename)
// ----------------------------------------------
Bmp::Bmp(const std::string& filename)
{
    assert(sizeof(UINT32) == 4);
    assert(sizeof(UINT16) == 2);

    data = 0;

    cout << "Reading " << filename << "\n";

    FILE *in = fopen(filename.c_str(), "rb");
    if (!in) throw IOError(filename);

    loaded_from_file = filename;

    char   buffer[BUFFERSIZE];
    size_t read;

#define fill_buffer(x) do{                      \
        read = fread(buffer, 1, x, in);         \
        assert(read<BUFFERSIZE);                \
        if (read != x) throw IOError(filename); \
        buffer[x] = 0;                          \
    }while(0)

    try {
        // -------------------------
        //      BITMAPFILEHEADER
        // -------------------------
        fill_buffer(2);
        if (strcmp(buffer, "BM") != 0) {
            wrong_format :
            throw Error("Expected Windows-BMP Format");
        }
        fill_buffer(4); UINT32 size = *(UINT32*)buffer; // filesize

        fill_buffer(2); if (buffer[0] || buffer[1]) goto wrong_format; // reserved1
        fill_buffer(2); if (buffer[0] || buffer[1]) goto wrong_format; // reserved2

        fill_buffer(4); UINT32 offset = *(UINT32*)buffer; // bitmap offset

#if defined(VERBOSE)
        cout << "size=" << size << " ";
        cout << "offset=" << offset << " ";
        cout << "\n";
#endif // VERBOSE

        // -------------------------
        //      BITMAPINFOHEADER
        // -------------------------

        fill_buffer(4); UINT32 infoSize        = *(UINT32*)buffer;
        fill_buffer(4); xsize                  = *(UINT32*)buffer;
        fill_buffer(4); ysize                  = *(UINT32*)buffer;
        fill_buffer(2); UINT16 planes          = *(UINT16*)buffer;
        fill_buffer(2); UINT16 bitsPerPixel    = *(UINT16*)buffer;
        fill_buffer(4); UINT32 compression     = *(UINT32*)buffer;
        fill_buffer(4); UINT32 dataSize        = *(UINT32*)buffer;
        fill_buffer(4); UINT32 X_pixPerMeter   = *(UINT32*)buffer;
        fill_buffer(4); UINT32 Y_pixPerMeter   = *(UINT32*)buffer;
        fill_buffer(4); UINT32 colorsUsed      = *(UINT32*)buffer;
        fill_buffer(4); UINT32 colorsImportant = *(UINT32*)buffer;

        USE(colorsImportant);
        USE(colorsUsed);
        USE(X_pixPerMeter);
        USE(Y_pixPerMeter);
        USE(infoSize);
        USE(size);
        USE(dataSize);

#if defined(VERBOSE)
        cout << "infoSize=" << infoSize << " ";
        cout << "xsize=" << xsize << " ";
        cout << "ysize=" << ysize << " ";
        cout << "\n";
        cout << "planes=" << planes << " ";
        cout << "bitsPerPixel=" << bitsPerPixel << " ";
        cout << "compression=" << compression << " ";
        cout << "dataSize=" << dataSize << " ";
        cout << "\n";
        cout << "X_pixPerMeter=" << X_pixPerMeter << " ";
        cout << "Y_pixPerMeter=" << Y_pixPerMeter << " ";
        cout << "\n";
        cout << "colorsUsed=" << colorsUsed << " ";
        cout << "colorsImportant=" << colorsImportant << " ";
        cout << "\n";
#endif // VERBOSE

        if (planes != 1 || bitsPerPixel != 24) throw Error("Can only read 24-bit graphics");
        if (compression != 0) throw Error("Can't read compressed BMPs");

        if (fseek(in, offset, SEEK_SET) != 0) throw IOError(filename);

        offset_per_line = 3*xsize;
        data            = new unsigned char[offset_per_line*ysize];

        size_t file_offset_per_line = ((offset_per_line-1)/4+1)*4;
        size_t xtra_offset          = file_offset_per_line-offset_per_line;

#if defined(VERBOSE)
        cout << "offset_per_line=" << offset_per_line << " ";
        cout << "file_offset_per_line=" << file_offset_per_line << " ";
        cout << "offset_per_line*ysize=" << offset_per_line*ysize << " ";
        cout << "\n";
#endif // VERBOSE
        assert(dataSize == 0 || dataSize == (file_offset_per_line*ysize));

        {
            unsigned char *lineStart = data+(ysize-1)*offset_per_line;
            int            lines     = ysize;

            while (lines--) {
                size_t bytes = fread(lineStart, 1, offset_per_line, in);
//                 cout << "read: offset_per_line=" << offset_per_line << " bytes=" << bytes << endl;
                if (bytes != offset_per_line) throw IOError(filename);

                if (xtra_offset>0) fill_buffer(xtra_offset);

                lineStart -= offset_per_line;
            }
        }

#if defined(VERBOSE)
        cout << filename << " in memory!\n";
#endif // VERBOSE
    }
//     catch (IOError& err) {
//         throw;
//     }
    catch (Error& err) {
        throw Error(string("While reading ")+filename+": "+err.what());
    }
    fclose(in);
#if defined(VERBOSE)
    cout << "--\n";
#endif // VERBOSE
}

// ----------------------------------------------------------------------------
//      inline void write_buffer(const char *buf, size_t bytes, FILE *out)
// ----------------------------------------------------------------------------
inline void write_buffer(const char *buf, size_t bytes, FILE *out) {
    size_t wrote = fwrite(buf, 1, bytes, out);
    if (wrote != bytes) throw IOError("write");
}

inline void write_UINT32(UINT32 ui, FILE *out) { write_buffer((char*)&ui, sizeof(ui), out); }
inline void write_UINT16(UINT16 ui, FILE *out) { write_buffer((char*)&ui, sizeof(ui), out); }

// ------------------------------------------------------------
//      void Bmp::saveAs(const std::string& filename) const
// ------------------------------------------------------------
void Bmp::saveAs(const std::string& filename) const
{
    cout << "Saving " << filename << "\n";
    FILE *out = fopen(filename.c_str(), "wb");
    if (!out) throw IOError(filename);

    try {
        // --------------------------
        //      BITMAPFILEHEADER:
        // --------------------------

        write_buffer("BM", 2, out); // filetype
        write_UINT32(0, out);   // file-size (@@@ calc!)
        write_UINT16(0, out);   // reserved1
        write_UINT16(0, out);   // reserved2
        write_UINT32(54, out);  // offset to bitmapstart

        // ----------------------------------
        //      BITMAPINFOHEADER:
        // ----------------------------------

        write_UINT32(40, out);  // size of BITMAPINFOHEADER
        write_UINT32(Xsize(), out); // xsize of bitmap
        write_UINT32(Ysize(), out); // ysize of bitmap
        write_UINT16(1, out);   // all data in one plane
        write_UINT16(24, out);   // 24-bit per pixel
        write_UINT32(0, out);   // no compression
        write_UINT32(0, out);   // size of compressed data
        write_UINT32(0, out);   // X_pixPerMeter
        write_UINT32(0, out);   // Y_pixPerMeter
        write_UINT32(0, out);   // colors used
        write_UINT32(0, out);   // colors important

        // ------------------
        //      bitmap :
        // ------------------


        {
            unsigned char *lineStart            = data+(ysize-1)*offset_per_line;
            int            lines                = ysize;
            size_t         file_offset_per_line = ((offset_per_line-1)/4+1)*4;
            size_t         xtra_offset          = file_offset_per_line-offset_per_line;
            char           xtra_buffer[4];

            for (int x = 0; x<4; ++x) xtra_buffer[x] = 0;

            while (lines--) {
                size_t bytes = fwrite(lineStart, 1, offset_per_line, out);
//                 cout << "read: offset_per_line=" << offset_per_line << " bytes=" << bytes << endl;
                if (bytes != offset_per_line) throw IOError(filename);

                if (xtra_offset>0) write_buffer(xtra_buffer, xtra_offset, out);

                lineStart -= offset_per_line;
            }
        }

    }
    catch (Error& err) {
        throw Error(string("While writing ")+filename+": "+err.what());
    }

    fclose(out);
}

// -----------------------------------------
//      void Bmp::throw_wrong_pix_pos()
// -----------------------------------------
void Bmp::throw_wrong_pix_pos()
{
    throw InternalError("Pixel position outside bitmap");
}

// -end- of implementation of class Bmp.

namespace rs {
    // --------------------------------------
    //      Bmp *halfBmp(const Bmp& bmp)
    // --------------------------------------
    Bmp *halfBmp(const Bmp& bmp) {
        int  hx   = bmp.Xsize()/2;
        int  hy   = bmp.Ysize()/2;
        Bmp *half = new Bmp(hx, hy);

        for (int x = 0; x<hx; ++x) {
            int x2 = x+x;
            for (int y = 0; y<hy; ++y) {
                int   y2 = y+y;
                Pixel p[4];
                p[0]     = bmp.getPixel(x2, y2);
                p[1]     = bmp.getPixel(x2+1, y2);
                p[2]     = bmp.getPixel(x2+1, y2+1);
                p[3]     = bmp.getPixel(x2, y2+1);

                int new_value[3];
                for (int c = 0; c<3; ++c) { // for all 3 colors
                    new_value[c] = 2; // this is equiv to round 'new_value[c]/4'
                    for (int pix = 0; pix<4; ++pix) { // of all 4 pixels
                        new_value[c] += p[pix].color(c); // build colors-sums
                    }
                }

                half->setPixel(x, y, new_value[0]/4, new_value[1]/4, new_value[2]/4);
            }
        }

        return half;
    }
};

