//  ======================================================================== //
//                                                                           //
//    File      : Date.cpp                                                   //
//    Purpose   :                                                            //
//    Time-stamp: <Wed Dec/26/2001 14:04 MET Coder@ReallySoft.de>            //
//                                                                           //
//    (C) December 2001 by Ralf Westram                                      //
//                                                                           //
//    Permission to use, copy, modify, distribute and sell this software     //
//    and its documentation for any purpose is hereby granted without fee,   //
//    provided that the above copyright notice appear in all copies and      //
//    that both that copyright notice and this permission notice appear      //
//    in supporting documentation.                                           //
//                                                                           //
//    Ralf Westram makes no representations about the suitability of this    //
//    software for any purpose.  It is provided "as is" without express or   //
//    implied warranty.                                                      //
//                                                                           //
//  ======================================================================== //

#ifndef CMEMORY
#include <cstring>
#endif
#ifndef CSTDIO
#include <cstdio>
#endif
#ifndef ERR_H
#include <Err.h>
#endif

#include "Date.h"

using namespace std;
using namespace rs;
using namespace rs::date;
using namespace rs::err;

// start of implementation of class Date:


// --------------------------------------------------
//      void Date::init(const std::time_t& time_)
// --------------------------------------------------
void Date::init(const std::time_t& time_)
{
    valid = false;
    time  = time_;
    if (time != -1) {
        date = *localtime(&time);
        init(date);
    }
}

// ----------------------------------------------
//      void Date::init(const std::tm& date_)
// ----------------------------------------------
void Date::init(const std::tm& date_)
{
    date  = date_;
    time  = mktime(&date);
    if (time != -1) {
        date = *localtime(&time);
        if (date_.tm_mday == date.tm_mday &&
            date_.tm_mon  == date.tm_mon &&
            date_.tm_year == date.tm_year)
        {
            valid = true;
        }
    }
}
// -----------------------------
//      void Date::correct()
// -----------------------------
void Date::correct()
{
    struct tm tmp = date;
    init(tmp);
}

// ---------------------------------------------------------
//      void Date::init(int day_, int month_, int year_)
// ---------------------------------------------------------
void Date::init(int day_, int month_, int year_)
{
    struct tm tmp;
    memset(&tmp, 0, sizeof(tmp));

    tmp.tm_mday = day_;
    tmp.tm_mon  = month_-1;
    tmp.tm_year = year_-1900;

    init(tmp);
}

// -------------------------
//      struct Feiertag
// -------------------------
struct Feiertag {
    int         day, month;
    const char *name;
    bool        catholic; // catholic Feiertag (phone is not cheaper)
    bool        half;
};

// ----------------------------
//      enum FeiertagIndex
// ----------------------------
enum FeiertagIndex {
    NO_FEIERTAG        = 0,
    FIRST              = 1,
    TAG_DER_DT_EINHEIT = 5,
    HL_ABEND           = 7,
    LAST_FIXED         = 10,
    OSTERSONNTAG,
    OSTERMONTAG,
    WEISSER_SONNTAG,
    CHRISTI_HIMMELFAHRT,
    PFINGSTSONNTAG,
    PFINGSTMONTAG,
    FRONLEICHNAM,
    KARFREITAG,
    PALMSONNTAG,
    BUSS_UND_BETTAG,
    ADVENT_1,
    ADVENT_2,
    ADVENT_3,
    ADVENT_4,
    LAST               = ADVENT_4
};

static int year_calculated_for = -1;

// --------------------------------------
//      static Feiertag feiertag[] =
// --------------------------------------
static Feiertag feiertag[] = {
    { 0,  0, "", false, false},
    { 1,  1, "Neujahr", false, false},
    { 6,  1, "Heilige drei Knige", true, false},
    { 1,  5, "Tag der Arbeit", false, false},
    {15,  8, "Maria Himmelfahrt", true, false},
    { 3, 10, "Tag der deutschen Einheit", false, false}, // (17.6 before 1990)
    { 1, 11, "Allerheiligen", false, false},
    {24, 12, "Heiliger Abend", false, true},
    {25, 12, "1. Weihnachtsfeiertag", false, false},
    {26, 12, "1. Weihnachtsfeiertag", false, false},
    {31, 12, "Silvester", false, true},

    // the following have variing dates:

    {0, 0, "Ostersonntag", false, false},
    {0, 0, "Ostermontag", false, false},
    {0, 0, "Weisser Sonntag", false, false},
    {0, 0, "Christi Himmelfahrt", false, false},
    {0, 0, "Pfingstsonntag", false, false},
    {0, 0, "Pfingstmontag", false, false},
    {0, 0, "Fronleichnam", false, false},
    {0, 0, "Karfreitag", false, false},
    {0, 0, "Palmsonntag", false, false},
    {0, 0, "Bu- und Bettag", false, false},
    {0, 0, "1. Advent", false, false},
    {0, 0, "2. Advent", false, false},
    {0, 0, "3. Advent", false, false},
    {0, 0, "4. Advent", false, false}
};

// --------------------------------------------------------------
//      inline bool matches(int feiertag_idx, const Date& d)
// --------------------------------------------------------------
inline bool matches(int feiertag_idx, const Date& d) {
    assert(feiertag_idx >= FIRST && feiertag_idx <= LAST);
    const struct tm& date  = d.get_Date();
    return
        date.tm_mday      == feiertag[feiertag_idx].day &&
        (date.tm_mon+1)   == feiertag[feiertag_idx].month;
}

// ------------------------------------------------------------
//      inline void store(const Date& d, int feiertag_idx)
// ------------------------------------------------------------
inline void store(const Date& d, int feiertag_idx) {
    assert(feiertag_idx > LAST_FIXED && feiertag_idx <= LAST);
    const struct tm& date = d.get_Date();

    feiertag[feiertag_idx].day = date.tm_mday;
    feiertag[feiertag_idx].month = date.tm_mon+1;
}

// -----------------------------------------------------
//      static void calculateFeiertageFor(int year)
// -----------------------------------------------------
static void calculateFeiertageFor(int year) {
    assert(year_calculated_for != year);
    year_calculated_for = year;

//     year -= 1900;

    int a = year%19;
    int b = year/100;
    int c = year%100;
    int d = b/4;
    int e = b%4;
    int f = (b+8)/25;
    int g = (b-f+1)/3;
    int h = (19*a+b-d-g+15)%30;
    int i = c/4;
    int k = c%4;
    int l = (32+2*e+2*i-h-k)%7;
    int m = (a+11*h+22*l)/451;

    int Easter = h+l-7*m+22;

//     year += 1900;

    feiertag[OSTERSONNTAG].month = Easter>31 ? 4 : 3;
    feiertag[OSTERSONNTAG].day   = Easter>31 ? Easter-31 : Easter;

    Date d1(feiertag[OSTERSONNTAG].day, feiertag[OSTERSONNTAG].month, year);
    Date d2(d1);

    d1.inc(); store(d1, OSTERMONTAG);
    d1.inc(6); store(d1, WEISSER_SONNTAG);
    d1.inc(32); store(d1, CHRISTI_HIMMELFAHRT);
    d1.inc(10); store(d1, PFINGSTSONNTAG);
    d1.inc(1); store(d1, PFINGSTMONTAG);
    d1.inc(10); store(d1, FRONLEICHNAM);

    d2.dec(2); store(d2, KARFREITAG);
    d2.dec(5); store(d2, PALMSONNTAG);

    Date d3(feiertag[HL_ABEND].day, feiertag[HL_ABEND].month, year);
    d3.dec(d3.dayOfWeek()); // last sunday before or at HL_ABEND

    store(d3, ADVENT_4);
    d3.dec(7); store(d3, ADVENT_3);
    d3.dec(7); store(d3, ADVENT_2);
    d3.dec(7); store(d3, ADVENT_1);
    d3.dec(11); store(d3, BUSS_UND_BETTAG);

    printf("year=%i\n", year);

    if (year<1990) {
        feiertag[TAG_DER_DT_EINHEIT].day   = 17;
        feiertag[TAG_DER_DT_EINHEIT].month = 6;
    }
    else {
        feiertag[TAG_DER_DT_EINHEIT].day   = 3;
        feiertag[TAG_DER_DT_EINHEIT].month = 10;
    }
}

// --------------------------------------
//      int Date::is_Feiertag() const
// --------------------------------------
int Date::is_Feiertag() const
{
    if (year_calculated_for != year()) {
        calculateFeiertageFor(year());
    }

    for (int idx = FIRST; idx <= LAST; ++idx) {
        if (matches(idx, *this)) return idx;
    }
    return NO_FEIERTAG;
}


// -end- of implementation of class Date.

// ------------------------------------------------------
//      static void testValidIndex(int feiertag_idx)
// ------------------------------------------------------
static void testValidIndex(int feiertag_idx) {
    if (feiertag_idx<FIRST || feiertag_idx>LAST) {
        throw Error("Illegal 'Feiertag' index");
    }
}

namespace rs {
    namespace date {
        // --------------------------------------------------------
        //      const char *name_of_Feiertag(int feiertag_idx)
        // --------------------------------------------------------
        const char *name_of_Feiertag(int feiertag_idx) {
            testValidIndex(feiertag_idx);
            return feiertag[feiertag_idx].name;
        }

        // --------------------------------------------
        //      bool is_catholic(int feiertag_idx)
        // --------------------------------------------
        bool is_catholic(int feiertag_idx) {
            testValidIndex(feiertag_idx);
            return feiertag[feiertag_idx].catholic;
        }
    };
};
