/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000
// Ralf Westram
// Time-stamp: <Thu Dec/26/2002 01:47 MET Coder@ReallySoft.de>
//
// Permission to use, copy, modify, distribute and sell this software
// and its documentation for any purpose is hereby granted without fee,
// provided that the above copyright notice appear in all copies and
// that both that copyright notice and this permission notice appear
// in supporting documentation.  Ralf Westram makes no
// representations about the suitability of this software for any
// purpose.  It is provided "as is" without express or implied warranty.
//
// This code is part of my library.
// You may find a more recent version at http://www.reallysoft.de/
//
/////////////////////////////////////////////////////////////////////////////

#define FILELIST_KENNUNG "RS_FILELIST"
#define FILELIST_VERSION 1

#include "Directory.h"

#include <dirent.h>
#include <unistd.h>
#include <list>

#include <Tools.h>
#include <sys_dep.h>
#include <read_lnk.h>

#ifdef DOS
#include <sys/cygwin.h>
// #include <windef.h>
// #define MAX_PATH 256 (using PATH_MAX instead)
#else
// #include <stdlib.h>
#endif


using namespace std;
using namespace rs;
using namespace rs::str;
using namespace rs::err;
using namespace rs::file;
using namespace rs::posix;

inline time_t min(time_t t1, time_t t2) { return difftime(t1, t2)<0 ? t1 : t2; }
inline time_t max(time_t t1, time_t t2) { return difftime(t1, t2)>0 ? t1 : t2; }

// start of implementation of class Directory:

// -----------------------------------------------------------------
//      string Directory::fullPath(const string& filename) const
// -----------------------------------------------------------------
string Directory::fullPath(const string& filename) const
{
    if (filename.rfind(DIR_SEPARATOR) == string::npos) { // no DIR_SEPARATOR
        return catDF(name(), filename);
    }
    return filename; // if filename already contains a path (happens i.e. with follow_links (see scanDir))
}

//  ------------------------------------------------------------------
//      void Directory::addFile(const string& filename, time_t t)
//  ------------------------------------------------------------------
void Directory::addFile(const string& filename, time_t t) {

//     if (filename.rfind(DIR_SEPARATOR) == string::npos); // don't insert paths

    if (files.empty()) {
        oldest_ = newest_ = t;
    }
    else {
        oldest_ = min(oldest_, t);
        newest_ = max(newest_, t);
    }
    files.insert(filename);
}

//  ----------------------------------------------------
//      void Directory::save(BinaryFile& out) const
//  ----------------------------------------------------
void Directory::save(BinaryFile& out) const {
    out.put_string(name_);
    out.put_nat(files.size());
    for (set<string>::const_iterator f=files.begin(); f != files.end(); ++f) {
        out.put_string(*f);
    }
    put_binary(oldest_, out);
    put_binary(newest_, out);
}
//  --------------------------------------------------------------
//      void Directory::load(BinaryFile& in, int /*version*/)
//  --------------------------------------------------------------
void Directory::load(BinaryFile& in, int /*version*/) {
    in.get_string(name_);
    unsigned long count;
    in.get_nat(count);
    files.clear();
    while (count--) {
        string s;
        in.get_string(s);
        files.insert(s);
    }
    get_binary(oldest_, in);
    get_binary(newest_, in);
}

// -end- of implementation of class Directory.

namespace rs {
    namespace file {

#ifdef DOS
        char DIR_SEPARATOR = '\\';
#else
        char DIR_SEPARATOR = '/';
#endif

        //  ----------------------------------------------------------------------
        //      string catDF(const string& directory, const string& filename)
        //  ----------------------------------------------------------------------
        string catDF(const string& directory, const string& filename) {
            return directory+DIR_SEPARATOR+filename;
            //             return directory+'/'+filename;
        }
        // ---------------------------------------------------
        //      void scanDir_internal(Directories&   dirs,
        // ---------------------------------------------------
        void scanDir_internal(Directories&   dirs,
                              const string&  rootdir,
                              bool           recurse,
                              bool           follow_links,
                              predicate      takeFile,
                              predicate      recurseDirectory,
                              ostream       *display,
                              set<string>&   visited_directories)
            // @@@ FIXME: all scanned directories should be inserted into visited_directories
            // and then repeated scanning should be avoided
        {
            Directory& dir = dirs[rootdir];
            dir.setName(rootdir);

            if (display) *display << rootdir << endl;

            DIR          *d = opendir(rootdir.c_str());
            list<string>  subdirs;
            list<string>  linked_dirs;

            if (!d) throw IOError(strf("opendir '%s' failed", rootdir.c_str()));
            while (1) {
                struct dirent *entry = readdir(d);
                if (!entry) break;

                const char *name = entry->d_name;
                if (name[0]=='.' && (name[1]==0 || (name[1]=='.' && name[2]==0))) continue; // '.' und '..'

                struct stat st;
                string      fullname = catDF(rootdir, entry->d_name);

                if (::stat(fullname.c_str(), &st) != 0) throw IOError(fullname, "stat failed");

                string linktarget;
                if (follow_links && isLink(fullname, linktarget)) {
                    struct stat st_target;
                    if (::stat(linktarget.c_str(), &st_target) != 0) {
                        throw IOError(linktarget, string("stat to target of '")+fullname+"' failed");
                    }

                    if (S_ISDIR(st_target.st_mode)) {
                        if (recurseDirectory(extractDirectory(linktarget).c_str(), extractFilename(linktarget).c_str(), st_target)) {
                            linked_dirs.push_back(linktarget);
                        }
                    }
                    else if (takeFile(extractDirectory(linktarget).c_str(), extractFilename(linktarget).c_str(), st_target)) {
                        dir.addFile(linktarget, st_target.st_mtime);
                    }
                }
                else if (S_ISDIR(st.st_mode)) {
                    if (recurseDirectory(rootdir, name, st)) {
                        subdirs.push_back(entry->d_name);
                    }
                }
                else if (takeFile(rootdir, name, st)) {
                    dir.addFile(name, st.st_mtime);
                }
            }

            closedir(d);

            if (!subdirs.empty() && recurse) {
                for (list<string>::const_iterator s=subdirs.begin(); s != subdirs.end(); ++s) {
                    scanDir(dirs, catDF(rootdir, *s), recurse, follow_links, takeFile, recurseDirectory, display);
                }
            }
        }

        void scanDir(Directories&   dirs,
                     const string&  rootdir,
                     bool           recurse,
                     bool           follow_links,
                     predicate      takeFile,
                     predicate      recurseDirectory,
                     ostream       *display)
        {
            set<string> visited_directories;
            scanDir_internal(dirs, rootdir, recurse, follow_links, takeFile, recurseDirectory, display, visited_directories);
        }

        //  ---------------------------------------------------------------------------------------------
        //      static bool recurseAnyDirectory(const char */*filename*/, const struct stat& /*st*/)
        //  ---------------------------------------------------------------------------------------------
        static bool recurseAnyDirectory(const string& /*dir*/, const char */*filename*/, const struct stat& /*st*/) {
            return true;
        }

        //  -----------------------------------------
        //      void scanDir(Directories&   dirs,
        //  -----------------------------------------
        void scanDir(Directories&   dirs,
                     const string&  rootdir,
                     bool           recurse,
                     bool           follow_links,
                     predicate      takeFile,
                     ostream       *display)
        {
            scanDir(dirs, rootdir, recurse, follow_links, takeFile, recurseAnyDirectory, display);
        }
        //  -----------------------------------------------------------------------
        //      void saveDirectories(const Directories& dirs, BinaryFile& out)
        //  -----------------------------------------------------------------------
        void saveDirectories(const Directories& dirs, BinaryFile& out) {
            out.put_string(FILELIST_KENNUNG);
            out.put_int(FILELIST_VERSION);

            out.put_nat(dirs.size());

            for (Directories::const_iterator d=dirs.begin(); d != dirs.end(); ++d) {

                const Directory& dir = d->second;
                dir.save(out);
            }
        }

        //  ----------------------------------------------------------------
        //      void loadDirectories(Directories& dirs, BinaryFile& in)
        //  ----------------------------------------------------------------
        void loadDirectories(Directories& dirs, BinaryFile& in) {
            string kennung;
            int version;

            in.get_string(kennung);
            if (kennung!=FILELIST_KENNUNG) throw Error(strf("Illegal filetype ('%s')", in.name().c_str()));

            in.get_int(version);

            unsigned long dcount;
            in.get_nat(dcount);
            while (dcount--) {
                Directory d;
                d.load(in, version);
                dirs[d.name()] = d;
            }
        }


        // -----------------------------------------------------
        //      string canonicalize(const string& filename)
        // -----------------------------------------------------
        string canonicalize(const string& filename, bool to_win32) {
//             string current          = getcwd();
//             string dir              = extractDirectory(filename);
//             if (chdir(dir.c_str()) != 0) throw IOError(filename, "canonicalize");

//             string file_dir             = getcwd();
//             if (chdir(current.c_str()) != 0) throw IOError(current, "canonicalize");

            //             string result = catDF(file_dir, extractFilename(filename));

            char buf[PATH_MAX+1];

            // @@@ FIXME: maybe canonicalize should have a second parameter specifying
            // whether to create a posix-path or a windows-path ?

#ifdef DOS
            if (to_win32) {
                cygwin_conv_to_full_win32_path(filename.c_str(), buf);
            }
            else {
                cygwin_conv_to_full_posix_path(filename.c_str(), buf);
            }
#else                           // LINUX
            // #error here a linux equivalent for cygwin_conv_to_full_win32_path is missing here.. sorry!
            if (to_win32) throw Error("canonicalize() does not support conversion to win32-pathname under Linux");
            realpath(filename.c_str(), buf);
#endif

            assert(buf[0] != 0); // otherwise it couldnt be converted
            string result = buf;

#if defined(DEBUG) && 0
            cout << "canonicalize(" << filename << ", " << to_win32 << ")='" << result << "'\n";
#endif // DEBUG
            return result;
        }

        // -----------------------------------------------------------------
        //      inline size_t lastPathSeperator(const string& filename)
        // -----------------------------------------------------------------
        inline size_t lastPathSeperator(const string& filename) {
            return filename.find_last_of("/\\");
        }

        // ---------------------------------------------------------
        //      string extractDirectory(const string& fullpath)
        // ---------------------------------------------------------
        string extractDirectory(const string& fullpath) {
            size_t slash  = lastPathSeperator(fullpath);
            string result = (slash == string::npos) ? "" : fullpath.substr(0, slash);
#if defined(DEBUG) && 0
            cout << "extractDirectory(" << fullpath << ")='" << result << "'\n";
#endif // DEBUG
            return result;
        }

        // --------------------------------------------------------
        //      string extractFilename(const string& fullpath)
        // --------------------------------------------------------
        string extractFilename(const string& fullpath) {
            size_t slash  = lastPathSeperator(fullpath);
            string result = (slash == string::npos) ? fullpath : fullpath.substr(slash+1);
#if defined(DEBUG) && 0
            cout << "extractFilename(" << fullpath << ")='" << result << "'\n";
#endif // DEBUG
            return result;
        }
#define BUFFERSIZE (PATH_MAX+1)

        // ----------------------------------------------------
        //      void backslashes2slashes(string& filename)
        // ----------------------------------------------------
        void backslashes2slashes(string& filename) {
            for (unsigned i=0; i<filename.length(); ++i) {
                if (filename[i]=='\\') {
                    filename[i] = '/';
                }
            }
        }

        // ---------------------------------------------------------
        //      bool isLink(const string& file, string& target)
        // ---------------------------------------------------------
        bool isLink(const string& file, string& target) {
            char buffer[BUFFERSIZE];
            buffer[0] = 0;

            bool   is_a_link = false;
            size_t lnk_pos   = file.find(".lnk");

            if (lnk_pos == file.length()-4) { // test for windows .lnk file
                Lnk lnk(file);

                if (lnk.points_to_file_or_directory()) {
                    target    = lnk.get_target();
                    is_a_link = true;
                }
            }

            if (!is_a_link) {
                // readlink does only work with unix
                // or with *.lnk files created using cygwin
                // (it does not work with 'real' windows links)

                string unixpath = canonicalize(file, false);
                int    result   = readlink(unixpath.c_str(), buffer, BUFFERSIZE);
                if (result != -1) {
                    buffer[result] = 0;
                    target         = canonicalize(buffer);
                    is_a_link      = true;
                }
#if defined(DEBUG) && 0
                else {
                    IOError err(unixpath, "readlink");
                    err.print(cerr);
                }
#endif // DEBUG
            }

            return is_a_link;
        }

    };
};

