//  ======================================================================== //
//                                                                           //
//    File      : FileFilter.cpp                                             //
//    Purpose   :                                                            //
//    Time-stamp: <Sat May/04/2002 16:51 MET Coder@ReallySoft.de>            //
//                                                                           //
//    (C) May 2002 by Ralf Westram                                           //
//                                                                           //
//    Permission to use, copy, modify, distribute and sell this software     //
//    and its documentation for any purpose is hereby granted without fee,   //
//    provided that the above copyright notice appear in all copies and      //
//    that both that copyright notice and this permission notice appear      //
//    in supporting documentation.                                           //
//                                                                           //
//    Ralf Westram makes no representations about the suitability of this    //
//    software for any purpose.  It is provided "as is" without express or   //
//    implied warranty.                                                      //
//                                                                           //
//  ======================================================================== //

#include "FileFilter.h"
#include <sys_dep.h>
#include <fstream>


using namespace std;
using namespace rs;
using namespace rs::str;
using namespace rs::sys_dep;

namespace rs {
    namespace file {
        namespace filter {

            // ----------------------------------------------------
            //      static bool isInputName(const string *arg)
            // ----------------------------------------------------
            static bool isInputName(const string *arg) {
                bool stdinRedirected = !is_a_TTY(stdin);
                return stdinRedirected ? arg==0 : arg!=0;
            }
            // -----------------------------------------------------
            //      static bool isOutputName(const string *arg)
            // -----------------------------------------------------
            static bool isOutputName(const string *arg) {
                bool stdoutRedirected = !is_a_TTY(stdout);
                return stdoutRedirected ? arg==0 : arg!=0;
            }

            // start of implementation of class FileFilter:

            // -------------------------------------------------------------------------------------------------
            //      void FileFilter::init(cmdline::CmdLineArgument& input, cmdline::CmdLineArgument& output)
            // -------------------------------------------------------------------------------------------------
            void FileFilter::init(cmdline::CmdLineArgument& input, cmdline::CmdLineArgument& output)
            {
                out       = 0;
                is_stdout = false;
                in        = input ? new InputFile(input.get()) : new InputFile();

                if (output) {
                    out       = new ofstream(output.get().c_str());
                    is_stdout = false;
                }
                else {
                    out       = &cout;
                    is_stdout = true;
                }
            }

            // -end- of implementation of class FileFilter.

            // start of implementation of class FilterArguments:

            // ----------------------------------------------------------------------------------------------
            //      FilterArguments::FilterArguments(const string& input_type, const string& output_type)
            // ----------------------------------------------------------------------------------------------
            FilterArguments::FilterArguments(const string& input_type, const string& output_type)
                : input("input", strf("name of %s-inputfile", input_type.c_str()).c_str(), isInputName)
                , output("output", strf("name of %s-outputfile", output_type.c_str()).c_str(), isOutputName)
                , arg_info("If you redirect I/O to/from a file, the corresponding filename may be omitted.")
            {
            }

            // -end- of implementation of class FilterArguments.

        };
    };
};
