//  ======================================================================== //
//                                                                           //
//    File      : FileFilter.h                                               //
//    Purpose   : Easy to use interface for programs which act as filter.    //
//    Time-stamp: <Wed Nov/13/2002 05:43 MET Coder@ReallySoft.de>            //
//                                                                           //
//    (C) May 2002 by Ralf Westram                                           //
//                                                                           //
//    Permission to use, copy, modify, distribute and sell this software     //
//    and its documentation for any purpose is hereby granted without fee,   //
//    provided that the above copyright notice appear in all copies and      //
//    that both that copyright notice and this permission notice appear      //
//    in supporting documentation.                                           //
//                                                                           //
//    Ralf Westram makes no representations about the suitability of this    //
//    software for any purpose.  It is provided "as is" without express or   //
//    implied warranty.                                                      //
//                                                                           //
//  ======================================================================== //

#ifndef FILEFILTER_H
#define FILEFILTER_H

#ifndef OPTION_H
#include <Option.h>
#endif
#ifndef INPUTFILE_H
#include <InputFile.h>
#endif

namespace rs {
    namespace file {

        /// Interface for programs which act as filter
        namespace filter {

            class FilterArguments {
            private:
                cmdline::OptionalCmdLineArgument input;
                cmdline::OptionalCmdLineArgument output;
                cmdline::CmdLineAnnotation arg_info;

            public:
                /** @brief declares OptionalCmdLineArgument's for input- and outputfile

                    Additionally a CmdLineAnnotation about redirection is displayed by infos().

                    @param input_type name of input data type (i.e. "HTML")
                    @param output_type name of %output data type
                    @note Instances of this class should be declared global
                */
                FilterArguments(const std::string& input_type, const std::string& output_type);
                virtual ~FilterArguments() {}

                /** @brief get the input argument
                    @return the OptionalCmdLineArgument defined for input filename
                 */
                cmdline::OptionalCmdLineArgument& get_input() { return input; }

                /** @brief get the %output argument
                    @return the OptionalCmdLineArgument defined for %output filename
                */
                cmdline::OptionalCmdLineArgument& get_output() { return output; }
            };

            //  -------------------------
            //      class FileFilter
            //  -------------------------
            class FileFilter {
            private:
                file::InputFile *in;
                std::ostream    *out;
                bool             is_stdout;

                void init(cmdline::CmdLineArgument& input, cmdline::CmdLineArgument& output);

            public:
                FileFilter(FilterArguments& filter) { init(filter.get_input(), filter.get_output()); }
                FileFilter(cmdline::CmdLineArgument& input, cmdline::CmdLineArgument& output) { init(input, output); }

                virtual ~FileFilter() {
                    delete in;
                    if (!is_stdout) delete out;
                }

                bool getLine(std::string& s) { return in->getLine(s); }
                void putLine(const std::string& s) { (*out) << s << "\n"; }
            };

        };
    };
}; // end of namespace rs

#else
#error FileFilter.h included twice
#endif // FILEFILTER_H

