/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000
// Ralf Westram
// (Coded@ReallySoft.de)
//
// Permission to use, copy, modify, distribute and sell this software
// and its documentation for any purpose is hereby granted without fee,
// provided that the above copyright notice appear in all copies and
// that both that copyright notice and this permission notice appear
// in supporting documentation.  Ralf Westram makes no
// representations about the suitability of this software for any
// purpose.  It is provided "as is" without express or implied warranty.
//
// This code is part of my library.
// You may find a more recent version at http://www.reallysoft.de/
//
/////////////////////////////////////////////////////////////////////////////

#include "Tools.h"
#include "Err.h"
#include <cstdio>
#include <new>

#ifdef __STRICT_ANSI__
#error Do not define __STRICT_ANSI__ - I need vsnprintf
#endif

using namespace std;

namespace rs {

    namespace str {

        // --------------------------------------------------------------------------------
        //     string vstrf(const char *format, va_list argPtr)
        // --------------------------------------------------------------------------------
        string vstrf(const char *format, va_list argPtr) {
            static size_t  buf_size = 256;
            static char   *buffer   = new char[buf_size];

            size_t length;
            while (1) {
                if (!buffer) {
                    assert(buffer); // to stop when debugging
                    throw std::bad_alloc();
                }

                length = vsnprintf(buffer, buf_size, format, argPtr);
                if (length < buf_size) break; // string fits into current buffer

                // otherwise resize buffer :
                buf_size += buf_size/2;
//                 cerr << "Reallocate vstrf-buffer to size=" << buf_size << endl;
                delete [] buffer;
                buffer    = new char[buf_size];
            }

            return string(buffer, length);
        }

        //  ---------------------------------------------
        //      string strf(const char *format, ...)
        //  ---------------------------------------------
        string strf(const char *format, ...) {
            va_list argPtr;
            va_start(argPtr, format);
            string result = vstrf(format, argPtr);
            va_end(argPtr);

            return result;
        }

    };

    namespace output {

        // --------------------------------------------------------------------------------
        //         void outf(ostream& out, const char *format, ...)
        // --------------------------------------------------------------------------------
        void outf(ostream& out, const char *format, ...) {
            va_list argPtr;
            va_start(argPtr, format);
            out << str::vstrf(format, argPtr);
            va_end(argPtr);
        }

//         // ----------------------------------------------------------------------------------------
//         //      void fixed_width_output(std::ostream& out, const std::string& s, size_t width)
//         // ----------------------------------------------------------------------------------------
//         void fixed_width_output(std::ostream& out, const std::string& s, size_t width) {
//             int rest = width-s.length();

//             if (rest>0) {
//                 out << string(rest, ' ');
//             }
//             out << s;
//         }

        // -----------------------------------------------------------------------------------------------------------------------
        //      void printIndented(std::ostream& out, unsigned indent, unsigned border, unsigned width, const std::string& s)
        // -----------------------------------------------------------------------------------------------------------------------
        void printIndented(std::ostream& out, unsigned indent, unsigned border, unsigned width, const std::string& s) {
            unsigned line_feed = s.find('\n');
            if (line_feed!=string::npos) {
                printIndented(out, indent, border, width, s.substr(0, line_feed));
                out << string(border+indent, ' ');
                printIndented(out, indent, border, width, s.substr(line_feed+1, string::npos));
                return;
            }

            unsigned rest_len = width-indent;
            unsigned len = s.length();
            unsigned start = 0;
            bool first = true;

            while (len>rest_len) {
                if (!first) {
                    out << string(border+indent, ' ');
                }

                unsigned end_pos = start+rest_len;
                unsigned last_space = s.rfind(' ', end_pos);
                unsigned last_pos;

                if (last_space==string::npos || last_space<=start) { // no space found
                    last_pos = end_pos;
                    end_pos++;
                }
                else {
                    last_pos = last_space-1;
                    end_pos = last_space+1;
                }

                out << s.substr(start, last_pos-start+1) << "\n";
                len -= (end_pos-start);
                start = end_pos;
                first = false;
            }

            if (!first) {
                out << string(border+indent, ' ');
            }
            out << s.substr(start, string::npos) << "\n";
        }

        // ------------------------------------------------------------------------------------------------------------------
        //      void printIndented(std::ostream& out, unsigned width, const std::string& left, const std::string& right)
        // ------------------------------------------------------------------------------------------------------------------
        void printIndented(std::ostream& out, unsigned width, const std::string& left, const std::string& right) {
            out << left << ' ';
            unsigned left_len = left.length()+1;
            printIndented(out, left_len, 0, width-left_len, right);
        }

    };
};
