#ifndef GENERIC_ALGO_H
#define GENERIC_ALGO_H

namespace rs {

    /** @brief Some generic algorithms
     */
    namespace gen_algo {

        /// swap(a, b) swaps the contents of a and b
        template<class T> inline void swap(T& a, T& b) { T temp = a; a = b; b = temp; }

        /// min(a, b) returns the minimum of a and b
        template <class T> T min(const T& t1, const T& t2) { return t1<t2 ? t1 : t2; }
        /// max(a, b) returns the maximum of a and b
        template <class T> T max(const T& t1, const T& t2) { return t2<t1 ? t1 : t2; }

        /// @if INTERNAL_DOC
        namespace internal {
            template<class T, int I, int J> struct Swap {
                static inline void compareAndSwap(T* array) {
                    if (array[I] > array[J]) swap<T>(array[I], array[J]);
                }
            };

            template<class T, int I, int J> struct Loop {
                static const bool go = (J <= I-2);
                static inline void loop(T* array) {
                    Swap<T,J,J+1>::compareAndSwap(array);
                    Loop<T, (go ? I : 0), (go ? (J+1) : 0)>::loop(array);
                }
            };
            template<class T> struct Loop<T,0,0> { static inline void loop(T*) {} };
        };
        /// @endif

        /** @brief sorts an array of T with size N using inlined bubble-sort
         */
        template<class T, int N> struct BubbleSort {
            static inline void sort(T* array) {
                internal::Loop<T, N-1, 0>::loop(array);
                BubbleSort<T, N-1>::sort(array);
            }
        };

        /// BubbleSort an array which contains one element
        template<class T> struct BubbleSort<T, 1> { static inline void sort(T*) {} };

    };

};

#else
#error generic_algo.h included twice
#endif // GENERIC_ALGO_H
