#ifndef GENERIC_MATH_H
#define GENERIC_MATH_H

#ifndef RSDEF_H
#include <rsdef.h>
#endif


namespace rs {
    /// Generic mathematical algorithms
    namespace gen_math {

        /// real big integer (currently a 64bit int for cygwin)
        typedef long double LongDouble;

        /** @brief Generic power calculation

            Power<X,Y>::value calculates the Yth power of X at compile time
        */
        template<int X, int Y> struct Power { static const LongDouble value = X*Power<X, Y-1>::value; };

        /// calculate the zero-power (for generic recursion)
        template<int X> struct Power<X, 0> { static const LongDouble value = 1; };

        /** @brief Generic factorial calculation

            Factorial<N>::value calculates factorial of N at compile time
        */
        template<int N> struct Factorial {static const LongDouble value = N * Factorial<N-1>::value;};

        /// Defines the base factorial (for generic recursion)
        struct Factorial<0> { static const LongDouble value = 1; };
    };

};

#else
#error generic_math.h included twice
#endif // GENERIC_MATH_H
