/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000
// Ralf Westram
// (Coded@ReallySoft.de)
//
// Permission to use, copy, modify, distribute and sell this software
// and its documentation for any purpose is hereby granted without fee,
// provided that the above copyright notice appear in all copies and
// that both that copyright notice and this permission notice appear
// in supporting documentation.  Ralf Westram makes no
// representations about the suitability of this software for any
// purpose.  It is provided "as is" without express or implied warranty.
//
// This code is part of my library.
// You may find a more recent version at http://www.reallysoft.de/
//
/////////////////////////////////////////////////////////////////////////////

#include "oem2ansi.h"

using namespace std;
using namespace rs;

// --------------------------------------------------------------------------------

/*

 ANSI - Zeichensatz

    032  !"#$%&'()*+,-./
    048 0123456789:;<=>?
    064 @ABCDEFGHIJKLMNO
    080 PQRSTUVWXYZ[\]^_
    096 `abcdefghijklmno
    112 pqrstuvwxyz{|}~
    128 
    144 
    160 
    176 
    192 
    208 
    224 
    240 

*/

// --------------------------------------------------------------------------------

namespace rs {
    namespace oem2ansi {

        static char oem_to_ansi_tab[] = "?'\"_+^??<?????`'\"\"--~??>????"
        "Ͼ"
        "ǎԐѥ噞"
        "ƄФ";

        static char ansi_to_oem_tab[] = "׃"
        "Ѫ+++"
        "+--+-+++---+i++_̯"
        "յݯ=";

        // --------------------------------------------------------------------------------
        //     unsigned char oem_to_ansi(unsigned char c)
        // --------------------------------------------------------------------------------
        unsigned char oem_to_ansi(unsigned char c)
        {
            return c<128 ? c : oem_to_ansi_tab[c-128];
        }

        // --------------------------------------------------------------------------------
        //     unsigned char ansi_to_oem(unsigned char c)
        // --------------------------------------------------------------------------------
        unsigned char ansi_to_oem(unsigned char c)
        {
            return c<128 ? c : ansi_to_oem_tab[c-128];
        }

        // --------------------------------------------------------------------------------
        //     string oem_to_ansi(const string& s)
        // --------------------------------------------------------------------------------
        string oem_to_ansi(const string& s) {
            string neu = s;

            for (string::iterator n=neu.begin(); n!=neu.end(); ++n) {
                *n = oem_to_ansi(*n);
            }

            return neu;
        }
        // --------------------------------------------------------------------------------
        //     string ansi_to_oem(const string& s)
        // --------------------------------------------------------------------------------
        string ansi_to_oem(const string& s) {
            string neu = s;

            for (string::iterator n=neu.begin(); n!=neu.end(); ++n) {
                *n = ansi_to_oem(*n);
            }

            return neu;
        }

    };
};
