/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000
// Ralf Westram
// (Coded@ReallySoft.de)
//
// Permission to use, copy, modify, distribute and sell this software
// and its documentation for any purpose is hereby granted without fee,
// provided that the above copyright notice appear in all copies and
// that both that copyright notice and this permission notice appear
// in supporting documentation.  Ralf Westram makes no
// representations about the suitability of this software for any
// purpose.  It is provided "as is" without express or implied warranty.
//
// This code is part of my library.
// You may find a more recent version at http://www.reallysoft.de/
//
/////////////////////////////////////////////////////////////////////////////

#ifndef OEM2ANSI_H
#define OEM2ANSI_H

#ifndef RSDEF_H
#include "rsdef.h"
#endif

namespace rs {

    /// Windows/OEM to ANSI character conversion
    namespace oem2ansi {

        /** @brief Windows to ANSI character conversion.
            @param c windows character
            @return ANSI character
        */
        unsigned char oem_to_ansi(unsigned char c);

        /** @brief Windows to ANSI string conversion.
            @param s string containing windows characters
            @return string containing ANSI characters
        */
        std::string oem_to_ansi(const std::string& s);

        /** @brief ANSI to Windows character conversion.
            @param c ANSI character
            @return windows character
        */
        unsigned char ansi_to_oem(unsigned char c);

        /** @brief ANSI to Windows string conversion.
            @param s string containing ANSI characters
            @return string containing windows characters
        */
        std::string ansi_to_oem(const std::string& s);

    };
};

#else
#error oem2ansi.h included twice
#endif // OEM2ANSI_H
