//  ======================================================================== //
//                                                                           //
//    File      : read_lnk.cpp                                               //
//    Purpose   :                                                            //
//    Time-stamp: <Mon Dec/02/2002 13:10 MET Coder@ReallySoft.de>            //
//                                                                           //
//    (C) August 2002 by Ralf Westram                                        //
//                                                                           //
//    Permission to use, copy, modify, distribute and sell this software     //
//    and its documentation for any purpose is hereby granted without fee,   //
//    provided that the above copyright notice appear in all copies and      //
//    that both that copyright notice and this permission notice appear      //
//    in supporting documentation.                                           //
//                                                                           //
//    Ralf Westram makes no representations about the suitability of this    //
//    software for any purpose.  It is provided "as is" without express or   //
//    implied warranty.                                                      //
//                                                                           //
//  ======================================================================== //

#if defined(DEBUG)
// #define DUMP_LINK_PARSE
#endif // DEBUG

#include <fstream>
#include <sys_dep.h>
#include <Tools.h>
#include <Err.h>
#include "read_lnk.h"

using namespace std;
using namespace rs;
using namespace rs::posix;
using namespace rs::str;
using namespace rs::err;

typedef unsigned long  dword;
typedef unsigned short word;

// ----------------------------------------------------------
//      inline word Word(const char *data, off_t offset)
// ----------------------------------------------------------
inline word Word(const char *data, off_t offset) {
    assert(sizeof(word) == 2); // if fails redef word above
    return *reinterpret_cast<const word*>(data+offset);
}
// ------------------------------------------------------------
//      inline dword Dword(const char *data, off_t offset)
// ------------------------------------------------------------
inline dword Dword(const char *data, off_t offset) {
    assert(sizeof(dword) == 4); // if fails redef dword above
    return *reinterpret_cast<const dword*>(data+offset);
}
// --------------------------------------------------------------
//      inline string String(const char *data, off_t offset)
// --------------------------------------------------------------
inline string String(const char *data, off_t offset) {
    word len = Word(data, offset);
    return string(data+offset+sizeof(len), len);
}
// ---------------------------------------------------------------
//      inline string String0(const char *data, off_t offset)
// ---------------------------------------------------------------
inline string String0(const char *data, off_t offset) {
    return string(data+offset);
}

// -----------------------------------------------
//      inline bool Bit(word w, unsigned bit)
// -----------------------------------------------
inline bool Bit(word w, unsigned bit) {
    assert(bit < 16);
    return w & (1<<bit);
}
// -------------------------------------------------
//      inline bool Bit(dword dw, unsigned bit)
// -------------------------------------------------
inline bool Bit(dword dw, unsigned bit) {
    assert(bit < 32);
    return dw & (1<<bit);
}


// start of implementation of class Lnk:

// ------------------------------------------
//      Lnk::Lnk(const string& filename_)
// ------------------------------------------
Lnk::Lnk(const string& filename_)
    : filename(filename_)
{
    off_t  datasize = getFileSize(filename);
    char  *data     = new char[datasize];

    {
        ifstream in(filename.c_str(), ios::in|ios::binary);
        in.read(data, datasize);
    }

    unsigned long flags     = Dword(data, 0x14);
    unsigned long file_attr = Dword(data, 0x18);

    bool shell_item_id_present      = Bit(flags, 0);
    points_to_file_or_directory_     = Bit(flags, 1);
    bool has_description            = Bit(flags, 2);
    bool has_relative_path          = Bit(flags, 3);
    bool has_working_directory      = Bit(flags, 4);
    bool has_command_line_arguments = Bit(flags, 5);
    bool has_custom_icon            = Bit(flags, 6);
    target_is_directory             = Bit(file_attr, 4);

#if defined(DUMP_LINK_PARSE)
    bool target_is_read_only           = Bit(file_attr, 0);
    bool target_is_hidden              = Bit(file_attr, 1);
    bool target_is_system              = Bit(file_attr, 2);
    bool target_is_volume_label        = Bit(file_attr, 3); // not possible
    bool target_has_archive_flag_set   = Bit(file_attr, 5);
    bool target_is_encrypted           = Bit(file_attr, 6);
    bool target_is_normal              = Bit(file_attr, 7);
    bool target_is_temporary           = Bit(file_attr, 8);
    bool target_is_sparse_file         = Bit(file_attr, 9);
    bool target_has_reparse_point_data = Bit(file_attr, 10);
    bool target_is_compressed          = Bit(file_attr, 11);
    bool target_is_offline             = Bit(file_attr, 12);
#endif // DUMP_LINK_PARSE

#if defined(DUMP_LINK_PARSE)
    cout << "----------------------------------------\n";
    cout << "shell_item_id_present=" << shell_item_id_present << "\n";
    cout << "points_to_file_or_directory=" << points_to_file_or_directory << "\n";
    cout << "has_description=" << has_description << "\n";
    cout << "has_relative_path=" << has_relative_path << "\n";
    cout << "has_working_directory=" << has_working_directory << "\n";
    cout << "has_command_line_arguments=" << has_command_line_arguments << "\n";
    cout << "has_custom_icon=" << has_custom_icon << "\n";
    cout << "----------------------------------------\n";
    cout << "target_is_read_only=" << target_is_read_only << "\n";
    cout << "target_is_hidden=" << target_is_hidden << "\n";
    cout << "target_is_system=" << target_is_system << "\n";
    cout << "target_is_volume_label=" << target_is_volume_label << "\n";
    cout << "target_is_directory=" << target_is_directory << "\n";
    cout << "target_has_archive_flag_set=" << target_has_archive_flag_set << "\n";
    cout << "target_is_encrypted=" << target_is_encrypted << "\n";
    cout << "target_is_normal=" << target_is_normal << "\n";
    cout << "target_is_temporary=" << target_is_temporary << "\n";
    cout << "target_is_sparse_file=" << target_is_sparse_file << "\n";
    cout << "target_has_reparse_point_data=" << target_has_reparse_point_data << "\n";
    cout << "target_is_compressed=" << target_is_compressed << "\n";
    cout << "target_is_offline=" << target_is_offline << "\n";
    cout << "----------------------------------------\n";
#endif // DUMP_LINK_PARSE

    off_t offset = 0x4c; // continue behind header
    if (shell_item_id_present) {
        off_t item_id_list_size = Word(data, offset);
#if defined(DUMP_LINK_PARSE)
        cout << "item_id_list_size=" << hex << item_id_list_size << "\n";
#endif // DUMP_LINK_PARSE

        off_t item_offset = offset+2;
        while (1) {
            word size    = Word(data, item_offset);
#if defined(DUMP_LINK_PARSE)
            cout << "item_offset=" << hex << item_offset << " size=" << hex << size << "\n";
#endif // DUMP_LINK_PARSE
            if (size == 0) break;
            item_offset += size;
        }

        offset += item_id_list_size+2;
    }

#if defined(DUMP_LINK_PARSE)
    cout << "start of file location table=" << hex << offset << "\n";
#endif // DUMP_LINK_PARSE
    dword fileloc_size = Dword(data, offset);
    if (!points_to_file_or_directory()) {
        assert(fileloc_size == 0);
        offset    += sizeof(fileloc_size);
    }
    else { // points_to_file_or_directory
        // dword fileloc_size2 = Dword(data, offset+0x04);
        dword volume_flags  = Dword(data, offset+0x08);

        bool on_local_volume   = Bit(volume_flags, 0);
        bool on_network_volume = Bit(volume_flags, 1);

        off_t  finalpath_offset = offset+Dword(data, offset+0x18);
        string finalpath        = String0(data, finalpath_offset);

#if defined(DUMP_LINK_PARSE)
        cout << "finalpath_offset=" << hex << finalpath_offset << "\n";
        cout << "finalpath='" << finalpath << "'\n";
#endif // DUMP_LINK_PARSE

        if (on_local_volume) {
            // off_t  local_volume_offset = offset+Dword(data, offset+0x0c);
            off_t  base_path_offset    = offset+Dword(data, offset+0x10);
            string base_path           = String0(data, base_path_offset);

#if defined(DUMP_LINK_PARSE)
            cout << "on_local_volume\n";
            // cout << "local_volume_offset=" << hex << local_volume_offset << "\n";
            cout << "base_path_offset=" << hex << base_path_offset << "\n";
            cout << "base_path='" << base_path << "'\n";
#endif // DUMP_LINK_PARSE

            target = base_path + finalpath;
        }
        else if (on_network_volume) {
            off_t  network_volume_offset = offset+Dword(data, offset+0x0c);
            off_t  network_share_offset  = network_volume_offset+Dword(data, network_volume_offset+0x08);
            string network_share_name    = String0(data, network_share_offset);
#if defined(DUMP_LINK_PARSE)
            cout << "on_network_volume\n";
            cout << "network_volume_offset=" << hex << network_volume_offset << "\n";
            cout << "network_share_offset=" << hex << network_share_offset << "\n";
            cout << "network_share_name='" << network_share_name << "'\n";
#endif                          // DUMP_LINK_PARSE

            target = network_share_name + finalpath;
        }

        offset += fileloc_size;
    }

    if (has_description) {
        string description = String(data, offset);
#if defined(DUMP_LINK_PARSE)
        cout << "description='" << description << "'\n";
#endif // DUMP_LINK_PARSE
        offset += description.length()+2;
    }

    if (has_relative_path) {
        string rel_path = String(data, offset);
#if defined(DUMP_LINK_PARSE)
        cout << "rel_path='" << rel_path << "'\n";
#endif // DUMP_LINK_PARSE
        offset += rel_path.length()+2;
    }

    if (has_working_directory) {
        string work_dir = String(data, offset);
#if defined(DUMP_LINK_PARSE)
        cout << "work_dir='" << work_dir << "'\n";
#endif // DUMP_LINK_PARSE
        offset += work_dir.length()+2;
    }

    if (has_command_line_arguments) {
        string command_line = String(data, offset);
#if defined(DUMP_LINK_PARSE)
        cout << "command_line='" << command_line << "'\n";
#endif // DUMP_LINK_PARSE
        offset += command_line.length()+2;
    }

    if (has_custom_icon) {
        string icon_file = String(data, offset);
#if defined(DUMP_LINK_PARSE)
        cout << "icon_file='" << icon_file << "'\n";
#endif // DUMP_LINK_PARSE
        offset += icon_file.length()+2;
    }
}

// -----------------------------------
//      bool Lnk::is_valid() const
// -----------------------------------
bool Lnk::is_valid() const
{
    return
        (points_to_directory() && dirExists(get_target()))
        ||
        (points_to_file() && fileExists(get_target()));
}

// -end- of implementation of class Lnk.

