//  ======================================================================== //
//                                                                           //
//    File      : read_lnk.h                                                 //
//    Purpose   :                                                            //
//    Time-stamp: <Mon Dec/02/2002 01:49 MET Coder@ReallySoft.de>            //
//                                                                           //
//    (C) August 2002 by Ralf Westram                                        //
//                                                                           //
//    Permission to use, copy, modify, distribute and sell this software     //
//    and its documentation for any purpose is hereby granted without fee,   //
//    provided that the above copyright notice appear in all copies and      //
//    that both that copyright notice and this permission notice appear      //
//    in supporting documentation.                                           //
//                                                                           //
//    Ralf Westram makes no representations about the suitability of this    //
//    software for any purpose.  It is provided "as is" without express or   //
//    implied warranty.                                                      //
//                                                                           //
//  ======================================================================== //

#ifndef READ_LNK_H
#define READ_LNK_H

#ifndef STRING
#include <string>
#endif


namespace rs {

    /// Determine target and validity of Win9x link files (*.lnk)
    class Lnk {
    private:
        std::string filename;
        std::string target;

        bool points_to_file_or_directory_;
        bool target_is_directory;

    public:
        /** @brief Examine a .lnk file
            @param filename_ the name of the .lmk file
        */
        Lnk(const std::string& filename_);
        virtual ~Lnk() {}

        /// return if link points to a target (may be non-existing)
        bool points_to_file_or_directory() const { return points_to_file_or_directory_; }

        /// return if link points to an existing target
        bool is_valid() const;

        /// return true if link points to a directory (may be a non-existing directory)
        bool points_to_directory() const { return points_to_file_or_directory() && target_is_directory; }

        /// return true if link points to a file (may be a non-existing file)
        bool points_to_file() const { return points_to_file_or_directory() && !target_is_directory; }

        /// return the full pathname of the target
        const std::string& get_target() const { return target; }
    };


}; // end of namespace rs

#else
#error read_lnk.h included twice
#endif // READ_LNK_H

