//  ======================================================================== //
//                                                                           //
//    File      : rs_assert.cpp                                              //
//    Purpose   :                                                            //
//    Time-stamp: <Wed May/08/2002 20:31 MET Coder@ReallySoft.de>            //
//                                                                           //
//    (C) May 2002 by Ralf Westram                                           //
//                                                                           //
//    Permission to use, copy, modify, distribute and sell this software     //
//    and its documentation for any purpose is hereby granted without fee,   //
//    provided that the above copyright notice appear in all copies and      //
//    that both that copyright notice and this permission notice appear      //
//    in supporting documentation.                                           //
//                                                                           //
//    Ralf Westram makes no representations about the suitability of this    //
//    software for any purpose.  It is provided "as is" without express or   //
//    implied warranty.                                                      //
//                                                                           //
//  ======================================================================== //

#include "rs_assert.h"

#include <Err.h>
#include <Tools.h>
#include <stdexcept>


using namespace std;
using namespace rs;

static bool              initialized   = false;
static terminate_handler old_terminate = 0;

using namespace rs;
using namespace rs::err;
using namespace rs::str;

namespace rs {
    // ---------------------------------------------
    //      void handle_exception(exception& e)
    // ---------------------------------------------
    void handle_exception(exception& e) {
        Error *error = dynamic_cast<Error*>(&e);

        if (error) {
            error->print(cerr);
        }
        else {
            try {
                reThrow(e);
            }
            catch (Error& E) { E.print(cerr); }
            catch (...) {
                InternalError E(strf("unknown exception: %s", e.what()));
                E.print(cerr);
            }
        }
    }
};

// --------------------------------------
//      static void real_terminate()
// --------------------------------------
static void real_terminate() {
#if defined(DEBUG)
    cerr << "Program terminating..\n";
#endif // DEBUG
    if (uncaught_exception()) {
        cerr << "An uncaught exception occurred.\n\n"
             << "Maybe you forgot to put the following code into your main():\n\n"
             << "try {\n"
             << " // your stuff\n"
             << " return EXIT_SUCCESS;\n"
             << "}\n"
             << "catch (exception& e) { handle_exception(e); }\n"
             << "catch (...) { Error e(\"Unknown exception\"); e.print(cerr); }\n"
             << "return EXIT_FAILURE;\n"
            ;
    }

    old_terminate();
}

// start of implementation of class Terminator:

// ---------------------------------
//      Terminator::Terminator()
// ---------------------------------
Terminator::Terminator()
{
    hassert(!initialized);
    initialized   = true;
    old_terminate = set_terminate(real_terminate);
}

// ----------------------------------
//      Terminator::~Terminator()
// ----------------------------------
Terminator::~Terminator()
{
    set_terminate(old_terminate);
}

// -end- of implementation of class Terminator.

