/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000
// Ralf Westram
// Time-stamp: <Wed Nov/13/2002 05:54 MET Coder@ReallySoft.de>
//
// Permission to use, copy, modify, distribute and sell this software
// and its documentation for any purpose is hereby granted without fee,
// provided that the above copyright notice appear in all copies and
// that both that copyright notice and this permission notice appear
// in supporting documentation.  Ralf Westram makes no
// representations about the suitability of this software for any
// purpose.  It is provided "as is" without express or implied warranty.
//
// This code is part of my library.
// You may find a more recent version at http://www.reallysoft.de/
//
/////////////////////////////////////////////////////////////////////////////

#ifndef RSDEF_H
#define RSDEF_H

#ifndef __STRING__
#include <string>
#endif
#ifndef __IOSTREAM__
#include <iostream>
#endif
#ifndef __CSTDARG__
#include <cstdarg>
#endif
#ifndef __CCTYPE__
#include <cctype>
#endif

// --------------------------------------------------------------------------------

namespace rs {

    /** @brief used by other lib-parts when printing a copyright info
        @return     name of programmer/company (whatever you like)
        @see        MAIL_CONTACT for how to customize this
     */
    std::string COPYRIGHT_BY();

    /** @brief used by other lib-parts when printing my email address

        @param      progname name of current  program (I use this to set the email-recipient to progname)
        @return     email address

        @warning COPYRIGHT_BY() and MAIL_CONTACT() are not included in the lib-binary.
        You'll have to define them by one of two ways:

        @li Change COPYRIGHT_BY and MAIL_CONTACT (in mail_contact.cpp) to fit your needs
        and build librs_mail_contact.a

        @li A more easy way to get it to work, is to insert the following block
        into each of your programs:

        @include Ex_mail_contact.cpp
    */
    std::string MAIL_CONTACT(const std::string& progname);
};

// --------------------------------------------------------------------------------

#if defined(DEBUG)
# define DUMP
#endif // DEBUG

#if !defined(FINAL_VERSION)
# define DUMP_FUNC
#endif

/// declare 'ostream& operator<<(ostream&, typ&)' from 'void typ::dump(ostream&)'
#define DECLARE_STREAM_OP(typ) friend inline std::ostream& operator<<(std::ostream& out, const typ &me) { me.dump(out); return out; }

/// dump corrent source location to stderr
#define DUMP_LINE() cerr << __FILE__ << ' ' << __LINE__ << endl

namespace rs {
    /// @if INTERNAL_DOC
    namespace internal {
        inline void use(const void*) {}
    };
    /// @endif
};

/// macro to avoid warnings about intentionally unused variables
#define USE(x) do { rs::internal::use((const void*)&(x)); } while(0)

// --------------------------------------------------------------------------------

/** @brief I put all my code into this namespace

 */
namespace rs {
    /// used by assert macros to print the assertion failure
    std::string assertionText(const char *bed, const char *file, int line);
};

// --------------------------------------------------------------------------------

#else
#error rsdef.h included twice
#endif // RSDEF_H
