/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000
// Ralf Westram
// Time-stamp: <Sun Jul/28/2002 01:48 MET Coder@ReallySoft.de>
//
// Permission to use, copy, modify, distribute and sell this software
// and its documentation for any purpose is hereby granted without fee,
// provided that the above copyright notice appear in all copies and
// that both that copyright notice and this permission notice appear
// in supporting documentation.  Ralf Westram makes no
// representations about the suitability of this software for any
// purpose.  It is provided "as is" without express or implied warranty.
//
// This code is part of my library.
// You may find a more recent version at http://www.reallysoft.de/
//
/////////////////////////////////////////////////////////////////////////////

#include "sys_dep.h"

#ifdef __STRICT_ANSI__
#error Cannot compile sys_dep.cpp with __STRICT_ANSI__ declared
#endif

#include <unistd.h>
#include <sys/stat.h>
#include <climits>
#include <cerrno>

#ifdef DOS

//  -------------------------
//      DOS includes:
//  -------------------------
# ifndef _SYS_UTIME_H
#  include <sys/utime.h>
# endif

#else

//  -----------------------
//      Linux includes:
//  -----------------------
# ifndef __UTIME_H__
#  include <utime.h>
# endif

#endif

#include <Err.h>
#include <Tools.h>
#include <Directory.h>

using namespace std;
using namespace rs;
using namespace rs::err;
using namespace rs::str;
// using namespace rs::file;
// using namespace rs::posix;

namespace rs {

    namespace sys_dep {
        //  --------------------------------------
        //      bool is_a_TTY(int filehandle)
        //  --------------------------------------
        bool is_a_TTY(int filehandle) {
            return isatty(filehandle);
        }
        //  -------------------------------
        //      bool is_a_TTY(FILE *f)
        //  -------------------------------
        bool is_a_TTY(FILE *f) {
            return is_a_TTY(fileno(f));
        }

        // ---------------------------
        //      string tempname()
        // ---------------------------
#ifdef DOS
        string tempname() {
            static int counter = 0;
            string     temp_name;
            while (1) {
                temp_name = strf("rstmp%03i", counter);
                if (!posix::fileExists(temp_name)) break;
                ++counter;
                if (counter>999) throw Error("Can't create temporary file");
            }

            return temp_name;
        }
#else                           // LINUX
        string tempname() {
            return tmpnam(0);
        }
#endif

        void sleep(unsigned milliseconds) {
            ::sleep((milliseconds+999)/1000);
        }
    };

    namespace posix {
        // ------------------------------------------------
        //      off_t getFileSize(const string& fname)
        // ------------------------------------------------
        off_t getFileSize(const string& fname) {
            struct stat st;
            if (stat(fname.c_str(), &st) != 0) throw IOError(fname, "stat");

            return st.st_size;
        }
        //  ---------------------------------------------------
        //      time_t getFileModTime(const string& fname)
        //  ---------------------------------------------------
        time_t getFileModTime(const string& fname) {
            struct stat st;
            if (stat(fname.c_str(), &st) != 0) throw IOError(fname, "stat");

            return st.st_mtime;
        }
        //  ---------------------------------------------------------------
        //      void setFileModTime(const string& fname, time_t mtime)
        //  ---------------------------------------------------------------
        void setFileModTime(const string& fname, time_t mtime) {
            struct stat st;
            if (stat(fname.c_str(), &st) != 0) throw IOError(fname, "stat");
            struct utimbuf ut;
            ut.actime  = st.st_atime;
            ut.modtime = mtime;

            if (utime(fname.c_str(), &ut) != 0) throw IOError(fname, "utime");
        }

        //  ---------------------------------------------
        //      bool fileExists(const string& fname)
        //  ---------------------------------------------
        bool fileExists(const string& fname) {
            struct stat st;
            if (stat(fname.c_str(), &st) != 0) return false;
            return S_ISREG(st.st_mode);
        }



#define BUFFERSIZE (PATH_MAX+1)

        // -------------------------
        //      string getcwd()
        // -------------------------
        string getcwd() {
            char  buffer[BUFFERSIZE];
            char *cwd = ::getcwd(buffer, BUFFERSIZE);

            if (!cwd) throw IOError("getcwd");

            string result;
            if (strsicmp(cwd, "/cygdrive/") == 0) result = strf("%c:%s", cwd[10], cwd+11);
            else   result                    = cwd;

#if defined(DEBUG) && 0
            cout << "getcwd()='" << result << "'\n";
#endif // DEBUG
            return result;
        }

        // ----------------------------------------------------------------------------
        //      void create_directory(const string& dirname, bool throw_if_exists)
        // ----------------------------------------------------------------------------
        void create_directory(const string& dirname, bool throw_if_exists) {
            int result = mkdir(dirname.c_str(), S_IREAD|S_IWRITE|S_IEXEC);
            if (result != 0 && ((errno != EEXIST) || throw_if_exists)) {
                throw IOError(dirname, "create_directory");
            }
        }

        // ---------------------------------------------
        //      bool dirExists(const string& dname)
        // ---------------------------------------------
        bool dirExists(const string& dname) {
            struct stat st;
            if (stat(dname.c_str(), &st) != 0) return false;
            return S_ISDIR(st.st_mode);
        }


#undef BUFFERSIZE

    };
};

