/////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2000
// Ralf Westram
// Time-stamp: <Thu Nov/14/2002 09:38 MET Coder@ReallySoft.de>
//
// Permission to use, copy, modify, distribute and sell this software
// and its documentation for any purpose is hereby granted without fee,
// provided that the above copyright notice appear in all copies and
// that both that copyright notice and this permission notice appear
// in supporting documentation.  Ralf Westram makes no
// representations about the suitability of this software for any
// purpose.  It is provided "as is" without express or implied warranty.
//
// This code is part of my library.
// You may find a more recent version at http://www.reallysoft.de/
//
/////////////////////////////////////////////////////////////////////////////

#ifndef SYS_DEP_H
#define SYS_DEP_H

#ifndef __STDIO_H__
#include <stdio.h>
#endif
#ifndef __STRING__
#include <string>
#endif
#ifndef CTIME
#include <ctime>
#endif

#ifndef DOS
# ifndef LINUX
#  error Either LINUX or DOS has to be defined.
# endif
#endif



namespace rs {

    /// Here I place system-dependent stuff (non-POSIX)
    namespace sys_dep {

        /// returns true if filehandle is a teletype
        bool is_a_TTY(int filehandle);
        /// returns true if FILE f is a teletype
        bool is_a_TTY(FILE *f);

        /// creates a unused filename (like tmpnam)
        std::string tempname();

        /** @brief sleep
            @param milliseconds time to sleep in milliseconds
        */
        void sleep(unsigned milliseconds);
    };

    /// Here I place all POSIX stuff
    namespace posix {

        /// gets the %file size
        off_t getFileSize(const std::string& fname);

        /// gets the %file modification time
        time_t getFileModTime(const std::string& fname);

        /// sets the %file modification time
        void setFileModTime(const std::string& fname, time_t mtime);

        /// returns true if the %file @p fname exists
        bool fileExists(const std::string& fname);

        /// returns true if the directory @p dname exists
        bool dirExists(const std::string& dname);

        /// returns the full path name of the current working directory
        std::string getcwd();

        /** @brief create a directory
            @param dirname name of directory to create
            @param throw_if_exists if true then throw an IOError when directory exists
            (defaults to false)
        */
        void create_directory(const std::string& dirname, bool throw_if_exists = false);
    };

}; // end of namespace rs

#else
#error sys_dep.h included twice
#endif // SYS_DEP_H
