//  ======================================================================== //
//                                                                           //
//    File      : user.cpp                                                   //
//    Purpose   :                                                            //
//    Time-stamp: <Wed Nov/13/2002 06:10 MET Coder@ReallySoft.de>            //
//                                                                           //
//    (C) February 2002 by Ralf Westram                                      //
//                                                                           //
//    Permission to use, copy, modify, distribute and sell this software     //
//    and its documentation for any purpose is hereby granted without fee,   //
//    provided that the above copyright notice appear in all copies and      //
//    that both that copyright notice and this permission notice appear      //
//    in supporting documentation.                                           //
//                                                                           //
//    Ralf Westram makes no representations about the suitability of this    //
//    software for any purpose.  It is provided "as is" without express or   //
//    implied warranty.                                                      //
//                                                                           //
//  ======================================================================== //

#include "user.h"
#include <Tools.h>
#include <rs_assert.h>
#include <Err.h>

using namespace std;
using namespace rs;
using namespace rs::str;
using namespace rs::err;

#define OUT cerr
#define IN  cin

namespace rs {
    namespace user {
        // --------------------------------------------------------------
        //      string ask(const string& question, bool allow_empty)
        // --------------------------------------------------------------
        string ask(const string& question, bool allow_empty) {
            string answer;
            if (question.length()) OUT << question << endl;
            while (1) {
                getline(IN, answer, '\n');
                if (allow_empty || answer.length()) break;
                OUT << "Enter your answer and press ENTER (or press Ctrl-C to abort)" << endl;
            };

            return answer;
        }

        // ------------------------------------------------------------------
        //      bool ask_yn(string question, const char *default_answer)
        // ------------------------------------------------------------------
        bool ask_yn(string question, const char *default_answer) {
            while (1) {
                string answer = ask(default_answer ? (question+" ["+default_answer+"]") : question, true);
                if (answer.empty() && default_answer) answer = default_answer;
                strlwr(answer);

                if (answer == "yes" || answer == "y") return true;
                if (answer == "no" || answer == "n") return false;
                question = "Please answer with yes or no (y/n)";
            }
        }
        // ------------------------------------------------------------
        //      int ask_number(string question, int low, int high)
        // ------------------------------------------------------------
        int ask_number(string question, int low, int high) {
            int num = 0;
            assert(low <= high);
            while (1) {
                string answer = ask(question, true);
                num           = atoi(answer.c_str());

                if (num >= low && num <= high) break;
                question = strf("Please enter a number between %i and %i", low, high);
            }
            return num;
        }
        // ---------------------------------------------------------------------------
        //      size_t choose(const string& title, const vector<string>& answers)
        // ---------------------------------------------------------------------------
        size_t choose(const string& title, const vector<string>& answers) {
            int count     = 1;
            int max_width = 0;

            for (vector<string>::const_iterator s = answers.begin(); s != answers.end(); ++s, ++count) {
                int width                      = 3+(count<10 ? 1 : (count<100 ? 2 : 3)) + s->length() + 1;
                if (width>max_width) max_width = width;
            }

            size_t size    = answers.size();
            int    rows    = size;
            int    columns = 1;

            int tty_rows    = 50; // @@@ how can I detect the real TTY size ?
            int tty_columns = 80;

            if (rows>(tty_rows-3)) {
                columns = tty_columns/max_width;
                rows    = size/columns + !!(size%columns);
            }

            OUT << title;
            for (count = 0; count < rows; ++count) {
                OUT << '\n';
                for (int c = 0; c<columns; ++c) {
                    unsigned idx = count + c*rows;
                    if (idx<size) {
                        OUT << strf("%-*s", max_width, strf("(%i) %s", idx+1, answers[idx].c_str()).c_str());
                    }
                }
            }

            OUT << endl;

            if (!size) throw InternalError("No choices to choose from");
            if (size == 1) {
                OUT << "[Only one choice => autoselected!]\n";
                return 1;
            }
            return ask_number("", 1, size);
        }
        // -----------------------------------------------------------------------------------
        //      size_t choose(const string& title, const string& answers, char separator)
        // -----------------------------------------------------------------------------------
        size_t choose(const string& title, const string& answers, char separator) {
            vector<string> a;
            size_t         start = 0;

            while (1) {
                size_t sep = answers.find_first_of(separator, start);
                if (sep == string::npos) {
                    a.push_back(answers.substr(start));
                    break;
                }
                a.push_back(answers.substr(start, sep-start));
                start = sep+1;
            }
            return choose(title, a);
        }
        // -------------------------------------------
        //      void pause(const string& message)
        // -------------------------------------------
        void pause(const string& message) {
            ask(message+" -- Press ENTER when ready", true);
        }
    };
};





