//  ======================================================================== //
//                                                                           //
//    File      : user.h                                                     //
//    Purpose   :                                                            //
//    Time-stamp: <Wed Nov/13/2002 06:08 MET Coder@ReallySoft.de>            //
//                                                                           //
//    (C) February 2002 by Ralf Westram                                      //
//                                                                           //
//    Permission to use, copy, modify, distribute and sell this software     //
//    and its documentation for any purpose is hereby granted without fee,   //
//    provided that the above copyright notice appear in all copies and      //
//    that both that copyright notice and this permission notice appear      //
//    in supporting documentation.                                           //
//                                                                           //
//    Ralf Westram makes no representations about the suitability of this    //
//    software for any purpose.  It is provided "as is" without express or   //
//    implied warranty.                                                      //
//                                                                           //
//  ======================================================================== //

#ifndef USER_H
#define USER_H

#ifndef STRING
#include <string>
#endif
#ifndef VECTOR
#include <vector>
#endif


namespace rs {
    /// simple user interaction for command line utils
    namespace user {

        /** Display message and wait for RETURN
         */
        void pause(const std::string& message);

        /** ask a question and return user input
            @param question This is printed to stdout
            @param allow_empty if true, empty input is accepted
            @return user input
        */
        std::string ask(const std::string& question, bool allow_empty = false);

        /** ask a question and expect yes/no/y/n as answer
            @param question This is printed to stdout
            @param default_answer When user pressed ENTER take this as answer.
            If @p default_answer sis 0, pressing ENTER is not allowed.
            @return true if user answered 'yes' or 'y'
        */
        bool ask_yn(std::string question, const char *default_answer = 0);

        /** ask to enter a number between low and high */
        int ask_number(std::string question, int low = INT_MIN, int high = INT_MAX);

        /** @brief ask user to select between several choices
            @param title printed at top
            @param answers the possible answers separated by separator
            @param separator 'answers' is separated by separator
            @return 1 for first answer, 2 for second, etc.

            @par Example:
            @code
            int answer = choose("(y)es or (n)o ? ", "y,n");
            if (answer == 1) cout << "The answer was 'y'\n";
            @endcode
        */
        size_t choose(const std::string& title, const std::string& answers, char separator = ',');

        /** @brief ask user to select between several choices
            @param title printed at top
            @param answers the possible answers
            @return 1 for first answer, 2 for second, etc.
        */
        size_t choose(const std::string& title, const std::vector<std::string>& answers);

    };
}; // end of namespace rs

#else
#error user.h included twice
#endif // USER_H

